<?php
/**
 * This class is serve as a RESTFul client to access to Jasperreport Server for report generation.
 * Two base URL from Jasperreport Server API are used in this class, they are:-
 * 1. j_spring_security_check for login and session ID retrieving
 * 2. reportExecutions for report generation and export
 *
 * @author Tan Tze Yee <tzeyee@sains.com.my>
 * @version 1.0 (2019-01-24)
 */
class jasperreports_server{
	/**
	 * cURL resource
	 *
	 * @access private
	 * @var resource
	 */
	private $ch;

	/**
	 * Jasperreport server login ID
	 *
	 * @access private
	 * @var string
	 */
	private $username;

	/**
	 * Jasperreport server login password
	 *
	 * @access private
	 * @var string
	 */
	private $userpassword;

	/**
	 * Jasperreport server URL
	 *
	 * @access private
	 * @var string
	 */
	private $url;

	/**
	 * Array of headers
	 *
	 * @access private
	 * @var string[]
	 */
	private $headers=[];

	/**
	 * Jasperreport server session ID
	 *
	 * @access private
	 * @var string
	 */
	private $jsession_id;

	/**
	 * Report's export ID
	 *
	 * @access private
	 * @var string
	 */
	private $export_id;

	/**
	 * Report's request ID
	 *
	 * @access private
	 * @var string
	 */
	private $request_id;
	
	/**
	 * Constructor
	 *
	 * @param string $p_url Jasperreport Server URL.
	 * @param string $p_username Jasperreport Server login user name.
	 * @param string $p_userpassword Jasperreport Server login password.
	 * @return VOID
	 */
	public function __construct($p_url, $p_username, $p_userpassword){
		$this->url = $p_url;
		$this->username = $p_username;
		$this->userpassword = $p_userpassword;
	}

	/**
	 * Initialise cURL as client to access to RESTFul API of Jasperreport Server.
	 * Set basic setting for cURL.
	 *
	 * @return VOID
	 */
	private function api_client_init(){
		$this->ch = curl_init();
		curl_setopt($this->ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($this->ch, CURLOPT_FOLLOWLOCATION, 1);
	}

	/**
	 * API access
	 *
	 * @param string $p_url Jasperreport Server URL.
	 *
	 * @return string Return the result from the Jasperreport Server API call
	 */
	private function api_access($p_url){
		curl_setopt($this->ch, CURLOPT_URL, $p_url);
		$v_return = curl_exec($this->ch);
		if(curl_errno($this->ch)){
			echo $p_url;
			echo "Curl error: " . curl_error($this->ch);
			exit(0);
		} 

		curl_close($this->ch);
		$this->ch = NULL;
		return $v_return;
	}

	/**
	 * Retrieve HTTP header information return from the API call.
	 * Header information are keep in class property "headers".
	 *
	 * @param string $p_curl cURL object.
	 * @param string $p_header HTTP header return from Jasperreport Server.
	 *
	 * @return integer Return the length of http header
	 */
	private function header_get($p_curl, $p_header){
		$v_len = strlen($p_header);
		$p_header = explode(':', $p_header, 2);
		if (count($p_header) < 2){ // ignore invalid headers
			return $v_len;
		}

		$v_name = strtolower(trim($p_header[0]));
		if (!array_key_exists($v_name, $this->headers)){
			$this->headers[$v_name] = [trim($p_header[1])];
		}else{
			$this->headers[$v_name][] = trim($p_header[1]);
		}

		return $v_len;
	}

	/**
	 * Authenticate and retrieve session ID for report generation and export API call.
	 * Assume the session ID name is "jsession_id" which is default naming.
	 * Session ID is keep in class property "jsession_id".
	 *
	 * @return integer Return the length of http header
	 */
	private function session_create(){
		$this->api_client_init();
	
		if(substr($this->url,-1)=="/"){
			$v_url = $this->url . "j_spring_security_check";
		}else{
			$v_url = $this->url . "/j_spring_security_check";
		}
		$v_url .= "?j_username=".$this->username."&j_password=".$this->userpassword;

		curl_setopt($this->ch, CURLOPT_HEADERFUNCTION, array($this,'header_get'));

		$v_return = $this->api_access($v_url);

		$v_session_id = '';
		if(is_array($this->headers) and isset($this->headers['set-cookie'])){
			foreach($this->headers['set-cookie'] as $v_val){
				if(strtoupper(substr($v_val, 0, 10)=="JSESSIONID")){
					$v_tmp = explode(";" , $v_val);
					$this->jsession_id = $v_tmp[0];
					return true;
					break;
				}
			}
		}

		return false;
	}

	/**
	 * Generate parameters in XML format for report generation.
	 *
	 * @param mixed $p_report_info Report generation information either in XML format or in array format.
	 * @return string Return XML string
	 */
	private function report_param_generate($p_report_info){
		if(is_array($p_report_info)){
			$v_xml_template = '<reportExecutionRequest>
								<reportUnitUri>##report_uri##</reportUnitUri>
								<async>false</async>
								<freshData>##fresh_data##</freshData>
								<saveDataSnapshot>##save_snap_shot##</saveDataSnapshot>
								<outputFormat>##format##</outputFormat>
								<interactive>true</interactive>
								<ignorePagination>false</ignorePagination>
								##parameters##
							</reportExecutionRequest>';

			$v_xml_str = "";
			if(isset($p_report_info['report_uri'])){
				$v_xml_str = str_ireplace('##report_uri##', $p_report_info['report_uri'], $v_xml_template);
			}else{
				return false;
			}
			if(isset($p_report_info['fresh_data'])){
				$v_xml_str = str_ireplace('##fresh_data##', $p_report_info['fresh_data'], $v_xml_str);
			}else{
				$v_xml_str = str_ireplace('##fresh_data##', 'false', $v_xml_str);
			}
			if(isset($p_report_info['save_snap_shot'])){
				$v_xml_str = str_ireplace('##save_snap_shot##', $p_report_info['save_snap_shot'], $v_xml_str);
			}else{
				$v_xml_str = str_ireplace('##save_snap_shot##', 'true', $v_xml_str);
			}
			if(isset($p_report_info['format'])){
				$v_xml_str = str_ireplace('##format##', $p_report_info['format'], $v_xml_str);
			}else{
				$v_xml_str = str_ireplace('##format##', 'pdf', $v_xml_str);
			}
			if(isset($p_report_info['parameters'])){
				$v_param = '';
				foreach($p_report_info['parameters'] as $v_field=>$v_value){
					$v_param .= '
								<reportParameter name="'.$v_field.'">
									<value>'.$v_value.'</value>
								</reportParameter>
								';
				}
				if($v_param!=''){
					$v_param = '
								<parameters>
								' . $v_param . '
								</parameters>
								';
					$v_xml_str = str_ireplace('##parameters##', $v_param, $v_xml_str);
				}else{
					$v_xml_str = str_ireplace('##parameters##', '', $v_xml_str);
				}
			}else{
				$v_xml_str = str_ireplace('##parameters##', '', $v_xml_str);
			}
		}else{
			$v_xml_obj = simplexml_load_string($p_report_info);
			if($v_xml_obj){
				$v_xml_str = $p_report_info;
			}else{
				return false;
			}
		}

		return $v_xml_str;
	}

	/**
	 * Generate report based on parameters provided.
	 *
	 * @param mixed $p_report_info Report generation information either in XML format or in array format.
	 * @return boolean Return status of API call
	 */
	private function report_generate($p_report_info){
		$v_create = $this->session_create();
		if(!$v_create){
			return false;
		}
		$this->api_client_init();

		$v_xml_str = $this->report_param_generate($p_report_info);
		if(!$v_xml_str){
			return false;
		}

		if(substr($this->url,-1)=="/"){
			$v_url = $this->url . "rest_v2/reportExecutions";
		}else{
			$v_url = $this->url . "/rest_v2/reportExecutions";
		}

		curl_setopt($this->ch, CURLOPT_CUSTOMREQUEST, 'POST');
		curl_setopt($this->ch, CURLOPT_POST, 1);
		curl_setopt($this->ch, CURLOPT_POSTFIELDS, $v_xml_str);
		curl_setopt($this->ch, CURLOPT_HTTPHEADER, array(
														"Accept-Encoding: gzip, deflate",
														"Connection: keep-alive",
														"Content-Type: Application/XML",
														"Cookie: " . $this->jsession_id,
													));
		$v_return = $this->api_access($v_url);
		$v_xml_doc = simplexml_load_string($v_return);

		if(!$v_xml_doc){
			return false;
		}else{
			$this->export_id = $v_xml_doc->exports->export->id;
			$this->request_id = $v_xml_doc->requestId;

			return true;
		}
	}

	/**
	 * Export report based on parameters provided.
	 *
	 * @param mixed $p_report_info Report generation information either in XML format or in array format.
	 * @return mixed Return content of report base on the format requested
	 */
	public function report_get($p_report_info){
		$v_generate = $this->report_generate($p_report_info);
		if(!$v_generate){
			return false;
		}

		$this->api_client_init();

		if(substr($this->url,-1)=="/"){
			$v_url = $this->url . 'rest_v2/reportExecutions/' . $this->request_id . '/exports/' .  $this->export_id . '/outputResource';
		}else{
			$v_url = $this->url . '/rest_v2/reportExecutions/' . $this->request_id . '/exports/' .  $this->export_id . '/outputResource';
		}

		curl_setopt($this->ch, CURLOPT_HTTPHEADER, array(
												"Accept-Encoding: gzip, deflate",
												"Connection: keep-alive",
												"Cookie: " . $this->jsession_id,
                                            ));

		return $this->api_access($v_url);
	}
}